/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.soap;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import org.objectweb.petals.component.framework.ComponentConfiguration;
import org.objectweb.petals.jbi.descriptor.JBIDescriptor;
import org.objectweb.petals.ws.notification.WsnManager;
import org.objectweb.petals.ws.notification.handlers.request.GetCurrentMessageHandler;
import org.objectweb.petals.ws.notification.handlers.request.SubscribeHandler;
import org.objectweb.petals.ws.notification.handlers.request.UnsubscribeHandler;

/**
 * The SOAP component context. The context is filled by the SU listener (adding
 * modules, service descriptions...) and used by the listeners.
 *
 * @author Christophe HAMERLING (chamerling) - eBMWebSourcing
 *
 */
public class SoapComponentContext {

    /**
     * The map of JBI descriptors. Key is the service unit name.
     */
    private Map<String, JBIDescriptor> jbiDescriptors;

    /**
     * The map of services.xml files. Key is the service unit name.
     */
    private Map<String, File> servicesDescriptors;

    /**
     * The requested modules to engage for the service client. Key is the
     * address, List is the QNames of the modules.
     */
    private Map<String, List<String>> modules;

    /**
     * The additional parameters associated to a service. Key is the address,
     * value is a string containing parameters
     */
    private Map<String, String> servicesParameters;

    /**
     * The web service notification manager
     */
    protected WsnManager wsnManager;

    /**
     * The component configuration information
     */
    private ComponentConfiguration componentConfiguration;

    private Logger logger;

    /**
     * Creates a new instance of {@link SoapComponentContext}
     *
     */
    public SoapComponentContext(Logger logger, ComponentConfiguration componentConfiguration) {
        this.logger = logger;
        this.componentConfiguration = componentConfiguration;
        this.jbiDescriptors = new HashMap<String, JBIDescriptor>();
        this.servicesDescriptors = new HashMap<String, File>();
        this.modules = new HashMap<String, List<String>>();
        this.servicesParameters = new HashMap<String, String>();
        this.wsnManager = new WsnManager(this.logger);

        // add handlers to manager
        SubscribeHandler subHandler = new SubscribeHandler();
        this.wsnManager.addHandler(subHandler);
        GetCurrentMessageHandler currentHandler = new GetCurrentMessageHandler();
        this.wsnManager.addHandler(currentHandler);
        UnsubscribeHandler unsubHandler = new UnsubscribeHandler();
        this.wsnManager.addHandler(unsubHandler);
    }

    /**
     * Add a {@link JBIDescriptor}
     *
     * @param address
     * @param jbiDescriptor
     */
    public void addJbiDescriptor(String serviceUnitName, JBIDescriptor jbiDescriptor) {
        if (serviceUnitName == null) {
            throw new IllegalArgumentException("Service unit name could not be null");
        }
        jbiDescriptors.put(serviceUnitName, jbiDescriptor);
    }

    /**
     * Get the JBI descriptor for the given address.
     *
     * @param address
     * @return the {@link JBIDescriptor} if found, else return null
     */
    public JBIDescriptor getJbiDescriptor(String serviceUnitName) {
        return jbiDescriptors.get(serviceUnitName);
    }

    /**
     * Remove the {@link JBIDescriptor} for the given address
     *
     * @param address
     */
    public void removeJbiDescriptor(String serviceUnitName) {
        if (jbiDescriptors != null) {
            jbiDescriptors.remove(serviceUnitName);
        }
    }

    /**
     * Add a {@link JBIDescriptor}
     *
     * @param address
     * @param jbiDescriptor
     */
    public void addServiceDescriptor(String serviceUnitName, File serviceDescriptor) {
        if (serviceUnitName == null) {
            throw new IllegalArgumentException("Service unit name could not be null");
        }
        this.servicesDescriptors.put(serviceUnitName, serviceDescriptor);
    }

    /**
     * Get the service descriptor as {@link File} of the given service unit if
     * available.
     *
     * @param address
     * @return the file (services.xml) or null if no service descriptor is
     *         available
     */
    public File getServiceDescriptor(String serviceUnitName) {
        return this.servicesDescriptors.get(serviceUnitName);
    }

    /**
     * Remove the {@link JBIDescriptor} for the given service unit
     *
     * @param address
     */
    public void removeServiceDescriptor(String serviceUnitName) {
        if (this.servicesDescriptors != null) {
            this.servicesDescriptors.remove(serviceUnitName);
        }
    }

    /**
     * Add a {@link JBIDescriptor}
     *
     * @param address
     * @param modulesList
     */
    public void addModules(String address, List<String> modulesList) {
        if (address == null) {
            throw new IllegalArgumentException("Address could not be null");
        }
        this.modules.put(address, modulesList);
    }

    /**
     * Get the modules for the given address.
     *
     * @param address
     * @return the modules of the given address. Should at least contains the
     *         addressing module
     */
    public List<String> getModules(String address) {
        return this.modules.get(address);
    }

    /**
     * Remove the {@link JBIDescriptor} for the given address
     *
     * @param address
     */
    public void removeModules(String address) {
        if (this.modules != null) {
            this.modules.remove(address);
        }
    }

    /**
     * Set parameters to a serviceAddress
     *
     * @param serviceAddress
     *            the service address
     * @param parameters
     *            the parameters to set
     */
    public void setServicePamameters(String serviceAddress, String parameters) {
        if (serviceAddress == null) {
            throw new IllegalArgumentException("serviceAddress could not be null");
        }
        this.servicesParameters.put(serviceAddress, parameters);
    }

    /**
     * Get the parameters for the given serviceAddress.
     *
     * @param serviceAddress
     *            the service address
     * @return a string containing the parameters of the given serviceAddress.
     */
    public String getServiceParameters(String serviceAddress) {
        return this.servicesParameters.get(serviceAddress);
    }

    /**
     * Remove the parameters for the given serviceAddress
     *
     * @param serviceAddress
     *            the service address
     */
    public void removeServiceParameters(String serviceAddress) {
        if (this.servicesParameters != null) {
            this.servicesParameters.remove(serviceAddress);
        }
    }

    /**
     * @return the notificationManager
     */
    public WsnManager getWsnManager() {
        return this.wsnManager;
    }

    /**
     * @param wsnManager
     *            the notificationManager to set
     */
    public void setWsnManager(WsnManager wsnManager) {
        this.wsnManager = wsnManager;
    }

    public ComponentConfiguration getComponentConfiguration() {
        return componentConfiguration;
    }

    public void setComponentConfiguration(ComponentConfiguration componentConfiguration) {
        this.componentConfiguration = componentConfiguration;
    }
}
